﻿#include "StdAfx.h"
#include "MainMenuControler.h"
#include "CommonFunction.h"
#include "resource.h"
#include "XHyperLink.h"
#include "ServerAddrConfig.h"
#include "ClientConfig.h"



CMainMenuControler::CMainMenuControler(CPaintManagerUI* pPaintManger)
:m_pPaintManger(pPaintManger)
{
	m_vecControls.clear();
	m_uClipBoardFormat = RegisterClipboardFormat(_T("MM_NTALK_ID"));
	pButtonLineDown = NULL;
	pButtonLineUp = NULL;
    m_pVetticalLayout = NULL;
    m_pDargedControl = NULL;
}

CMainMenuControler::~CMainMenuControler()
{
	m_vecControls.clear();
}

bool CMainMenuControler::LoadActiveMenus()
{
	CMenuOptionUI* pMenuOption = NULL;

	pButtonLineDown = static_cast<CButtonUI*>(m_pPaintManger->FindControl(kMenuArrowDown));
	pButtonLineUp = static_cast<CButtonUI*>(m_pPaintManger->FindControl(kMenuArrowUp));

	ASSERT(pButtonLineDown);
	ASSERT(pButtonLineUp);
	if(pButtonLineDown == NULL || pButtonLineUp == NULL)
	{
		return false;
	}

	pButtonLineUp->SetEnabled(false);

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoChat)		
		 _LoadChatHistoryUI();

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoNote)		
		_LoadMemoUI();

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoMail)		
		_LoadEmailUI();

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSign)		
		_LoadApproveUI();

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC)		
		_LoadVOCUI();

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay)	
		_LoadSurveyUI();

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoIN)		
		_LoadINUI();

	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM)		
		_LoadSAMUI();

	//if (GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg)	;
	


	//"+" button
	pMenuOption = CreateMenu(L"file='btn_main_icon_more_dim.png'", 
		L"file='btn_main_icon_more_dim.png'",
		L"img_main_menu_bg_sel.png",
		LoadStringFromResource(IDS_STRING686),_T(""));
	pMenuOption->SetName(kMenuOptionAdd);
	pMenuOption->SetToolTip(LoadStringFromResource(IDS_STRING_MAIN_TOOLTIP_12));
	pMenuOption->SetEnabled(false);

	return true;
}


CButtonUI* CMainMenuControler::CreateButton( LPCTSTR strBKImage, LPCTSTR strText,
											LPCTSTR strName, LPCTSTR strTooltip )
{
	CButtonUI* pButton = new CButtonUI();
	pButton->SetAttribute(L"width", L"58");
	pButton->SetAttribute(L"height", L"44");

	pButton->SetBkImage(strBKImage);
	pButton->SetName(strName);
	pButton->SetToolTip(strTooltip);
	m_pVetticalLayout->Add(pButton);
	return pButton;
}

CMenuOptionUI* CMainMenuControler::CreateMenu( LPCTSTR strForeImage, LPCTSTR strSelectForeImage, 
							  LPCTSTR strSelectBKImage, LPCTSTR strText, 
							  LPCTSTR strSelectArrowImage )
{
	//CDialogBuilder dialogBuilder;
	CMenuOptionUI* pMenuOptions = new CMenuOptionUI();
	pMenuOptions->SetGroup(L"123");
	pMenuOptions->SetAttribute(L"width", L"58");
	pMenuOptions->SetAttribute(L"height", L"44");
	pMenuOptions->SetForeImage(strForeImage);
	pMenuOptions->SetSelectedForeImage(strSelectForeImage);
	pMenuOptions->SetSelectedArrowImage(strSelectArrowImage);
	pMenuOptions->SetSelectedImage(strSelectBKImage);

	

	/*pMenuOptions->SetText(strText);
	pMenuOptions->SetTextColor(0x9897a6);
	pMenuOptions->SetAttribute(L"align", L"center");
	pMenuOptions->SetAttribute(L"textpadding", L"0,33");
	//pMenuOptions->SetTextPadding(CRect(0,0,0,0));
	pMenuOptions->SetFont(4);*/

	/*CDuiString strTipLeftImageValue;
	strTipLeftImageValue.Format(L"file='img_main_menu_badge_icon_l.png' dest='%d,6,%d,18'",
		TIPLEFTIMAGELEFTPOS, TIPLEFTIMAGELEFTPOS + TIPIMAGEWIDTH);
	pMenuOptions->SetTipLeftImage(strTipLeftImageValue);
	pMenuOptions->SetTipCenterImage(L"file='img_main_menu_badge_icon_c.png'");
	pMenuOptions->SetTipRightImage(L"file='img_main_menu_badge_icon_r.png'");
	pMenuOptions->SetTipTextCorlor(0xFFFFFF);
	pMenuOptions->SetTipText(L"57");
	pMenuOptions->SetTipTextFont(3);*/

	m_pVetticalLayout->Add(pMenuOptions);
	return pMenuOptions;
}

void CMainMenuControler::SetLayout( CVerticalLayoutUI* pVetticalLayout )
{
	m_pVetticalLayout = pVetticalLayout;
}

bool CMainMenuControler::InitFixedMenus()
{
	/*CMenuOptionUI* pTabORG = static_cast<CMenuOptionUI*>(m_pPaintManger->FindControl(_T("tab_organization")));
	if(NULL == pTabORG) return false;
	pTabORG->SetText(LoadStringFromResource(IDS_STRING680));

	CMenuOptionUI* pTabBM = static_cast<CMenuOptionUI*>(m_pPaintManger->FindControl(_T("tab_bookmark")));
	if(NULL == pTabBM) return false;
	pTabBM->SetText(LoadStringFromResource(IDS_STRING681));

	CMenuOptionUI* pTabCL = static_cast<CMenuOptionUI*>(m_pPaintManger->FindControl(_T("tab_chatlist")));
	if(NULL == pTabCL) return false;
	pTabCL->SetText(LoadStringFromResource(IDS_STRING682));*/

	return true;

}

void CMainMenuControler::DragStart( TNotifyUI& msg )
{
	CPoint pt = msg.ptMouse;
	m_pDargedControl = HitTest(pt);
	if(m_pDargedControl == NULL) 
	{
		return;
	}

	OutputDebugString(_T("----DragStart----\n"));
	CSharedFile file;
	CArchive ar(&file, CArchive::store);

	int nIndex = m_pVetticalLayout->GetItemIndex(m_pDargedControl);
	ar<<nIndex;
	ar.Close();

	COleDataSource oleDataSource;
	oleDataSource.CacheGlobalData(m_uClipBoardFormat, file.Detach());
	oleDataSource.DoDragDrop();

}

CControlUI* CMainMenuControler::HitTest( CPoint pt )
{
	int nCount = m_pVetticalLayout->GetCount();

	//the nCount - 1 is "+"
	for (int i = 0; i < nCount - 1; i++)
	{
		CControlUI* pControl = m_pVetticalLayout->GetItemAt(i);
		CRect rc = pControl->GetPos();
		if (rc.PtInRect(pt))
		{
			return pControl;
		}
	}

	return NULL;
}

DWORD CMainMenuControler::GetClipBoardFormat()
{
	return m_uClipBoardFormat;
}

DROPEFFECT CMainMenuControler::OnDragEnter( CWnd* , COleDataObject* pDataObject, DWORD dwKeyState, CPoint )
{
	return DROPEFFECT_MOVE;
}

DROPEFFECT CMainMenuControler::OnDragOver( CWnd* pWnd , COleDataObject* pDataObject, DWORD dwKeyState, CPoint point )
{
	CControlUI* pControl = HitTest(point);
	int nEnd = m_pVetticalLayout->GetCount() - 1;
	if(pControl != NULL && m_pVetticalLayout->GetItemIndex(pControl) != nEnd)
	{
		return DROPEFFECT_MOVE;
	}
	return DROPEFFECT_NONE;
}

DROPEFFECT CMainMenuControler::OnDragScroll( CWnd* pWnd, DWORD dwKeyState, CPoint point )
{
	return DROPEFFECT_MOVE;
}

void CMainMenuControler::OnDragLeave()
{
	return;
}

BOOL CMainMenuControler::OnDrop( CWnd* , COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point )
{
	CControlUI* pControlTarget = HitTest(point);
	ASSERT(pControlTarget);
	if(NULL == pControlTarget)
	{
		return FALSE;
	}
	if(pDataObject)
	{
		if(pDataObject->IsDataAvailable(m_uClipBoardFormat))
		{
			CFile* pFile = pDataObject->GetFileData(m_uClipBoardFormat);
			CArchive ar(pFile, CArchive::load); 
			int nDragSourceIndex = -1;
			ar >> nDragSourceIndex;
			ar.Close();
			SAFE_DELETE(pFile);
			
			int nDragTargetIndex = m_pVetticalLayout->GetItemIndex(pControlTarget);
			CControlUI* pControlSource = m_pVetticalLayout->GetItemAt(nDragSourceIndex);
			m_pVetticalLayout->Swap(nDragSourceIndex, nDragTargetIndex);
			_SwapControl(pControlSource, pControlTarget);

		}
		pDataObject->Release();
		pDataObject = NULL;
	}


	return TRUE;
}



bool CMainMenuControler::_CheckAuth()
{
	return true;
}	

bool CMainMenuControler::_LoadMemoUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_message_nor.png' dest='14,7,44,37'",
		LoadStringFromResource(IDS_STRING683), kMenuOptionMemo, 
		LoadStringFromResource(IDS_STRING_MAIN_NOTE));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);

	return true;
}

bool CMainMenuControler::_LoadChatHistoryUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_chathistory_nor.png' dest='14,7,44,37'",
		LoadStringFromResource(IDS_STRING685), kMenuOptionChatHistory,
		LoadStringFromResource(IDS_STRING_MAIN_TOOLTIP_11));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);
	return true;
}

bool CMainMenuControler::_LoadEmailUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_mail_nor.png' dest='14,7,44,37'",
		_T(""), kMenuOptionMail,
		LoadStringFromResource(IDS_STRING_MAIN_MAIL));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);
	return true;

}

bool CMainMenuControler::_LoadApproveUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_approve_nor.png' dest='14,7,44,37'",
		_T(""), kMenuOptionApprove,
		LoadStringFromResource(IDS_STRING_MAIN_APPROVAL));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);
	return true;
}

bool CMainMenuControler::_LoadVOCUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_VOC_nor.png' dest='14,7,44,37'",
		_T(""), kMenuOptionVOC,
		_T("VOC"));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);
	return true;
}

bool CMainMenuControler::_LoadSurveyUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_survey_nor.png' dest='14,7,44,37'",
		_T(""), kMenuOptionSurvey,
		LoadStringFromResource(IDS_STRING_MAIN_SURVEY));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);
	return true;
}

bool CMainMenuControler::_LoadINUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_IN_nor.png' dest='14,7,44,37'",
		_T(""), kMenuOptionIN,
		LoadStringFromResource(IDS_STRING378));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);
	return true;
}

bool CMainMenuControler::_LoadSAMUI()
{
	if(!_CheckAuth())
		return false;

	CButtonUI* pButton = NULL;
	pButton = CreateButton(L"file='btn_main_icon_SAM_nor.png' dest='14,7,44,37'",
		_T(""), kMenuOptionSAM,
		LoadStringFromResource(IDS_STRING374));

	ASSERT(pButton);
	if(NULL == pButton)
		return false;

	m_vecControls.push_back(pButton);
	return true;
}

void CMainMenuControler::LineDown()
{
	CControlUI* pControl = NULL;
	int nCount = m_vecControls.size();
	for(vector<CControlUI*>::iterator it = m_vecControls.begin();
		it != m_vecControls.end(); it++)
	{
		pControl = *it;
		ASSERT(pControl);
		if(pControl != NULL && pControl->IsVisible())
		{
			pControl->SetVisible(false);
			if(!pButtonLineUp->IsEnabled())
			{
				pButtonLineUp->SetEnabled(true);
			}
			break;
		}
		else
		{
			continue;
		}	
	}

	//can't line down any more
	if(nCount > 0 && m_vecControls.at(nCount - 2) == pControl)
	{
		pButtonLineDown->SetEnabled(false);
	}
}

void CMainMenuControler::LineUp()
{
	CControlUI* pControl = NULL;
	int nCount = m_vecControls.size();
	for(vector<CControlUI*>::reverse_iterator reverseIT = m_vecControls.rbegin();
		reverseIT != m_vecControls.rend(); reverseIT++)
	{
		pControl = *reverseIT;
		ASSERT(pControl);
		if(pControl != NULL && !pControl->IsVisible())
		{
			pControl->SetVisible(true);
			if(!pButtonLineDown->IsEnabled())
			{
				pButtonLineDown->SetEnabled(true);
			}
			break;
		}
		else
		{	
			continue;
		}
	}

	// can't line up any more
	if(nCount > 0 && m_vecControls.at(0) == pControl)
	{
		pButtonLineUp->SetEnabled(false);
	}
}

bool CMainMenuControler::_SwapControl( CControlUI* pControlSource, CControlUI* pControlTarget )
{
	ASSERT(pControlSource && pControlTarget);
	if(NULL == pControlSource || NULL == pControlTarget)
	{
		return false;
	}

	vector<CControlUI*>::iterator itSource = std::find(m_vecControls.begin(), m_vecControls.end(), pControlSource);
	vector<CControlUI*>::iterator itTarget = std::find(m_vecControls.begin(), m_vecControls.end(), pControlTarget);
	if(itSource == m_vecControls.end() || itTarget == m_vecControls.end())
	{
		return false;
	}

	iter_swap(itSource, itTarget);

	return true;
		
}

void CMainMenuControler::SetInfoItem(NTHanatourCountType eType, int nCount, CString strURL){

	HANATOURBTNINFO info;
	info.count = nCount;
	info.strURL = strURL;

	std::map<NTHanatourCountType, HANATOURBTNINFO>::iterator it = m_mapHanatourBtnInfo.find(eType);
	if (it != m_mapHanatourBtnInfo.end())
	{
        m_mapHanatourBtnInfo.erase(it);
	} 
	m_mapHanatourBtnInfo.insert(std::make_pair(eType, info));
}

int CMainMenuControler::GetInfoItemCount(NTHanatourCountType eType){

	std::map<NTHanatourCountType, HANATOURBTNINFO>::iterator it = m_mapHanatourBtnInfo.find(eType);
	if (it != m_mapHanatourBtnInfo.end())
	{
		return it->second.count;
	} 
	return 0;
}

CString CMainMenuControler::GetInfoItemURL(NTHanatourCountType eType){

	std::map<NTHanatourCountType, HANATOURBTNINFO>::iterator it = m_mapHanatourBtnInfo.find(eType);
	if (it != m_mapHanatourBtnInfo.end())
	{
		return it->second.strURL;
	} 
	return _T("");
}